#!/usr/bin/env python
#
# HY-menu for few links and places
# Author: Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 22.1.2015

import os
import re
import gtk
import gio
import signal
import subprocess
import appindicator
import urllib
import commands
import subprocess
import pwd

APP_NAME = 'HY-menu'
APP_VERSION = '0.1'
username = pwd.getpwuid(os.getuid()).pw_name

class IndicatorPlaces:
    BOOKMARKS_PATH = '/etc/cubbli/hy-menu/hy-menu-items'

    def __init__(self):
        self.ind = appindicator.Indicator("places", "/etc/cubbli/hy-menu/hy-menu.png", appindicator.CATEGORY_APPLICATION_STATUS)
        self.ind.set_status(appindicator.STATUS_ACTIVE)
        self.fspath = self.get_fspath()
        
        self.update_menu()

    def create_menu_item(self, label, icon_name):
        image = gtk.Image()
        image.set_from_icon_name(icon_name, 24)

        item = gtk.ImageMenuItem()
        item.set_label(label)
        item.set_image(image)
        item.set_always_show_image(True)
        return item
    
    # Icons for different menu items
    def get_bookmark_icon(self, path):
        if path.startswith("smb") or path.startswith("ftp") or path.startswith("file"):
            icon_name = "folder-remote"
        elif path.endswith("helpdesk"):
            icon_name = "hint"
        elif path.endswith("Default.aspx"):
            icon_name = "mail_new"
        elif path.startswith("http://wpr"):
            icon_name = "stock_print"    
        else:
            f = gio.File(path)
            try:   
                info = f.query_info(gio.FILE_ATTRIBUTE_STANDARD_ICON)
                icon = info.get_icon()
                icon_name = icon.get_names()[0] if icon.get_names()[0] != '(null)' else 'folder'
            except (gio.Error, NameError):
                icon_name = "folder"
        
        return icon_name

    # This finds out which NFS-server to use
    def get_fspath(self):

        if re.match('[a-e]', username):
            home = '1'
        elif re.match('[f-j]', username):
            home = '2'
        elif re.match('[k-l]', username):
            home = '3'
        elif re.match('[m-o]', username):
            home = '4'
        elif re.match('[p-s]', username):
            home = '5'
        elif re.match('[t-z]', username):
            home = '6'
        else:
            assert False,"Invalid username!"

        return "/home/ad/fshome"+ home +"/u"+ home +"/"
    
    # This creates the menu
    def update_menu(self, widget = None, data = None):
        try:
            bookmarks = open(self.BOOKMARKS_PATH).readlines()
        except IOError:
            bookmarks = []        

        # Create menu
        menu = gtk.Menu()
        self.ind.set_menu(menu)

        # Home folder menu item
        item = self.create_menu_item("Home Folder", "user-home") 
        item.connect("activate", self.on_bookmark_click, os.getenv('HOME'))
        menu.append(item)

        # Computer info - Not ready yet
#        item = self.create_menu_item("Computer Info", "info")
#        item.connect("activate", self.on_bookmark_click, InfoWin())
#        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # Add bookmarks to the menu from the /etc/cubbli/hy-menu-items
        for bm in bookmarks:
            path, label = bm.strip().partition(' ')[::2]

            if not label:
                label = os.path.basename(os.path.normpath(path))

            label = urllib.unquote(label)
            item = self.create_menu_item(label, self.get_bookmark_icon(path))
            item.connect("activate", self.on_bookmark_click, path)

            # Append the item to menu
            menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # This creates Z-drive menu item
        item = self.create_menu_item("Z-drive", "user-home")
        item.connect("activate", self.on_bookmark_click, self.fspath + "/" + username[:1] + "/" + username)
        menu.append(item)

        # Show the menu
        menu.show_all()

    # Open clicked bookmark
    def on_bookmark_click(self, widget, path):
        if path.startswith("/home/ad/"):
            if not os.path.ismount(self.fspath):
                os.system("mount "+ self.fspath)
        #subprocess.Popen('/usr/bin/nautilus %s' % path, shell = True)
        subprocess.Popen('/usr/bin/xdg-open %s' % path, shell = True)
        
    def on_bookmarks_changed(self, filemonitor, file, other_file, event_type):
        if event_type == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
            print 'Bookmarks changed, updating menu...'
            self.update_menu()
           
if __name__ == "__main__":
    # Catch CTRL-C
    signal.signal(signal.SIGINT, lambda signal, frame: gtk.main_quit())

    # Run the indicator
    i = IndicatorPlaces()
    
    # Monitor bookmarks changes 
    file = gio.File(i.BOOKMARKS_PATH)
    monitor = file.monitor_file()
    monitor.connect("changed", i.on_bookmarks_changed)            
    
    # Main gtk loop
    gtk.main()
